@@ -1,6 +1,6 @@
-# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
+# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
 # SPDX-License-Identifier: Apache-2.0
-
+from datetime import datetime
 from typing import (
     TYPE_CHECKING,
     AbstractSet,
@@ -12,16 +12,19 @@ from typing import (
     Sequence,
     Union,
 )
+import uuid
 
 from ..damlast.daml_lf_1 import TypeConName
 from ..damlast.lookup import parse_type_con_name
-from ..prim import ContractData, ContractId, Party
+from ..prim import LEDGER_STRING_REGEX, ContractData, ContractId, Party, to_parties
 from ..util.typing import safe_cast
 
 __all__ = [
+    "ApplicationMeteringReport",
     "ArchiveEvent",
     "Boundary",
     "Command",
+    "CommandMeta",
     "CreateAndExerciseCommand",
     "CreateCommand",
     "CreateEvent",
@@ -30,8 +33,10 @@ __all__ = [
     "ExerciseByKeyCommand",
     "ExerciseCommand",
     "ExerciseResponse",
+    "ParticipantMeteringReport",
     "PartyInfo",
     "SubmitResponse",
+    "User",
 ]
 
 
@@ -321,6 +326,75 @@ class ExerciseByKeyCommand(Command):
         return f"ExerciseByKeyCommand({self.template_id}, {self.key}, {self.choice!r}, {self.argument})"
 
 
+class CommandMeta:
+    """
+    Additional fields that accompany a command submission.
+
+    .. py:attribute:: workflow_id
+        :type: str | None
+
+        An optional workflow ID.
+
+    .. py:attribute:: command_id
+        :type: str | None
+
+        An optional command ID. If unspecified, a random one will be created.
+
+    .. py:attribute:: read_as
+        :type: Collection[Party] | None
+
+        An optional set of read-as parties to use to submit this command. Note that for a
+        ledger with authorization, these parties must be a subset of the parties in the token.
+
+    .. py:attribute:: act_as
+        :type: Collection[Party] | None
+
+        An optional set of act-as parties to use to submit this command. Note that for a
+        ledger with authorization, these parties must be a subset of the parties in the token.
+    """
+
+    __slots__ = "workflow_id", "command_id", "read_as", "act_as"
+
+    if TYPE_CHECKING:
+        workflow_id: "Optional[str]"
+        command_id: "Optional[str]"
+        read_as: "Optional[Sequence[Party]]"
+        act_as: "Optional[Sequence[Party]]"
+
+    def __init__(
+        self,
+        workflow_id: "Optional[str]",
+        command_id: "Optional[str]",
+        read_as: "Union[None, Party, Collection[Party]]",
+        act_as: "Union[None, Party, Collection[Party]]",
+    ):
+        if workflow_id:
+            if not LEDGER_STRING_REGEX.match(workflow_id):
+                raise ValueError("workflow_id must be a valid ledger string")
+        else:
+            workflow_id = None
+
+        if command_id:
+            if not LEDGER_STRING_REGEX.match(command_id):
+                raise ValueError("command_id must be a valid ledger string")
+        else:
+            command_id = uuid.uuid4().hex
+
+        object.__setattr__(self, "workflow_id", workflow_id)
+        object.__setattr__(self, "command_id", command_id)
+        object.__setattr__(self, "read_as", to_parties(read_as))
+        object.__setattr__(self, "act_as", to_parties(act_as))
+
+    def __eq__(self, other: Any) -> bool:
+        return (
+            isinstance(other, CommandMeta)
+            and self.workflow_id == other.workflow_id
+            and self.command_id == other.command_id
+            and self.read_as == other.read_as
+            and self.act_as == other.act_as
+        )
+
+
 class CreateEvent:
     """
     An event that indicates a newly-created contract.
@@ -513,6 +587,22 @@ class ExerciseResponse:
 SubmitResponse = Union[None, CreateEvent, ExerciseResponse]
 
 
+class User:
+    """
+    Full information about a ``User``.
+
+    Note: This is part of a Daml 2.x pre-release API and is subject to change.
+    """
+
+    __slots__ = ("id", "primary_party")
+    id: str
+    primary_party: Party
+
+    def __init__(self, id: str, primary_party: Party):
+        self.id = id
+        self.primary_party = primary_party
+
+
 class PartyInfo:
     """
     Full information about a ``Party``.
@@ -551,6 +641,36 @@ class PartyInfo:
         return self._is_local
 
 
+class ParticipantMeteringReport:
+    __slots__ = ("report_generation_time", "participant_id", "to_actual", "application_reports")
+    report_generation_time: datetime
+    participant_id: str
+    to_actual: datetime
+    application_reports: "Sequence[ApplicationMeteringReport]"
+
+    def __init__(
+        self,
+        report_generation_time: datetime,
+        participant_id: str,
+        to_actual: datetime,
+        application_reports: "Sequence[ApplicationMeteringReport]",
+    ):
+        object.__setattr__(self, "report_generation_time", report_generation_time)
+        object.__setattr__(self, "participant_id", participant_id)
+        object.__setattr__(self, "to_actual", to_actual)
+        object.__setattr__(self, "application_reports", application_reports)
+
+
+class ApplicationMeteringReport:
+    __slots__ = ("application_id", "event_count")
+    application_id: str
+    event_count: int
+
+    def __init__(self, application_id: str, event_count: int):
+        object.__setattr__(self, "application_id", application_id)
+        object.__setattr__(self, "event_count", event_count)
+
+
 def validate_template_id(value: Union[str, TypeConName]) -> TypeConName:
     if isinstance(value, TypeConName):
         return value
